%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%AIRFOIL PROGRAM%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% ABOUT %%%

%This program will facilitate the design of airfoils and the exportation of
%airfoil designs to '.txt' files that may be read into SolidWorks as XYZ
%Curves. 

%From Wikipedia: "The NACA airfoils are airfoil shapes for aircraft wings 
%developed by the National Advisory Committee for Aeronautics (NACA). 
%The shape of the NACA airfoils is described using a series of digits 
%following the word "NACA". The parameters in the numerical code can be 
%entered into equations to precisely generate the cross-section of the 
%airfoil and calculate its properties."

%Notes 2/25/14 1729 EST
%There are a lot of bugs in the program. Here is a list of features that
%need to be fixed, changed, or added:

%1: Fix the plotting of the red camber line. It never plots properly when
%rotated, and I'm not even sure it plots correctly even without rotation.

%2: Add a section to the beginning of the plotbutton callback function that
%clears all the airfoils from the screen. 

%3: Logic check the rotational matrix. The rotation is about the origin; is
%that what you want? Think carefully about how the rotation 
%will affect its relation to the other airfoil. Another option would be to
%   a) Write a function that determines the geographical center of the
%   airfoil.
%   b) Write/modify the rotational funtion to rotate the airfoil about its
%   geographical center.
%But again, there are still issues with this method. Do some research on
%how wings are actually designed.

%4: Add a section for translating the airfoils in the plane. This should be simple; you are
%just adding to the x and y components of the vector. Perhaps a spinoff of
%this would be to define lines connecting the geographical centers of the
%airfoils to each other, or perhaps the leading and trailing edges?

%5: Finish the section that exports the airfoils. Perhaps in later versions
%there will be a drop down menu to select the CAD/CAE software the file is
%to be used for, and will format and save the coordinates in the
%corresponding format for that software.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% SETUP %%%

function airfoilGUI()

clear
clc

figure_h = figure(...
    'Visible','off',...
    'Units','normalized',...
    'Position',[0.15 0.20 0.8 0.85],...
    'Name','Airfoil Program',...
    'Color',[0.8 0.8 0.8]...
    );

movegui(figure_h,'center')

% clear figure window
clf

%=========================================================================%

%Define positions.

xCenter=.5;
yCenter=.5;
graphlength=.4;
graphheight=.2;
xGraph=xCenter-.5*graphlength;
yGraph=1-1.4*graphheight;
xButton=.05;
xPButton=.8;
buttonheight=.04;
ybuttonspace=1.5*buttonheight;
yButton=yGraph;
buttonlength=.08;
xbuttonspace=.1;


%=========================================================================%

%Define child-figures (axes, text boxes, edit boxes, drop-down menus,
%buttons, etc.).

%-------------------------------------------------------------------------%

%Airfoil Axes

%Airfoil 1 Axes
plot1_h = axes(...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'Position',[xGraph yGraph graphlength graphheight]...
    );

%Airfoil 2 Axes
plot2_h = axes(...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'Position',[xGraph yGraph-(1.4*graphheight) graphlength graphheight]...
    );

%Planform View Axes
plot3_h = axes(...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'Position',[xGraph yGraph-(3*graphheight) graphlength graphheight]...
    );
%-------------------------------------------------------------------------%

%NACA Airfoil Edit Boxes & Labels

%NACA Edit Box 1 Label
NACA1txt_h = uicontrol(...
    'Style','text',...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'BackgroundColor',[0.87 0.87 0.87],...
    'Position',[xButton yButton buttonlength buttonheight],...
    'FontSize',10,...
    'String','Airfoil 1: NACA Number'...
    );

%NACA Edit Box 1
NACA1edit_h = uicontrol(...
    'Style','edit',...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'Position',[xButton yButton-ybuttonspace buttonlength buttonheight],...
    'FontSize',10,...
    'String',''...
    );



%NACA Edit Box 2 Label
NACA2txt_h = uicontrol(...
    'Style','text',...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'BackgroundColor',[0.87 0.87 0.87],...
    'Position',[1-(buttonlength+xButton) yButton buttonlength buttonheight],...
    'FontSize',10,...
    'String','Airfoil 2: NACA Number'...
    );

%NACA Edit Box 2
NACA2edit_h = uicontrol(...
    'Style','edit',...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'Position',[1-(buttonlength+xButton) yButton-ybuttonspace buttonlength buttonheight],...
    'FontSize',10,...
    'String',''...
    );

%-------------------------------------------------------------------------%

%Chord Length Edit Boxes & Labels

%Chord Length Edit Box 1 Label
chord1txt_h = uicontrol(...
    'Style','text',...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'BackgroundColor',[0.87 0.87 0.87],...
    'Position',[xButton yButton-2*ybuttonspace buttonlength buttonheight],...
    'FontSize',10,...
    'String','Airfoil 1: Chord Length (in)'...
    );

%Chord Length Edit Box 1
chord1edit_h = uicontrol(...
    'Style','edit',...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'Position',[xButton yButton-3*ybuttonspace buttonlength buttonheight],...
    'FontSize',10,...
    'String',''...
    );


%Chord Length Edit Box 2 Label
chord2txt_h = uicontrol(...
    'Style','text',...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'BackgroundColor',[0.87 0.87 0.87],...
    'Position',[1-(buttonlength+xButton) yButton-2*ybuttonspace buttonlength buttonheight],...
    'FontSize',10,...
    'String','Airfoil 2: Chord Length (in)'...
    );

%Chord Length Edit Box 2
chord2edit_h = uicontrol(...
    'Style','edit',...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'Position',[1-(buttonlength+xButton) yButton-3*ybuttonspace buttonlength buttonheight],...
    'FontSize',10,...
    'String',''...
    );

%-------------------------------------------------------------------------%

%AOI Edit Boxes & Labels

%AOI Edit Box 1 Label
AOI1txt_h = uicontrol(...
    'Style','text',...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'BackgroundColor',[0.87 0.87 0.87],...
    'Position',[xButton yButton-4*ybuttonspace buttonlength buttonheight],...
    'FontSize',10,...
    'String','Airfoil 1: Angle Of Incidence'...
    );

%AOI Edit Box 1
AOI1edit_h = uicontrol(...
    'Style','edit',...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'Position',[xButton yButton-5*ybuttonspace buttonlength buttonheight],...
    'FontSize',10,...
    'String',''...
    );


%AOI Edit Box 2 Label
AOI2txt_h = uicontrol(...
    'Style','text',...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'BackgroundColor',[0.87 0.87 0.87],...
    'Position',[1-(buttonlength+xButton) yButton-4*ybuttonspace buttonlength buttonheight],...
    'FontSize',10,...
    'String','Airfoil 2: Angle Of Incidence'...
    );

%AOI Edit Box 2
AOI2edit_h = uicontrol(...
    'Style','edit',...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'Position',[1-(buttonlength+xButton) yButton-5*ybuttonspace buttonlength buttonheight],...
    'FontSize',10,...
    'String',''...
    );

%-------------------------------------------------------------------------%

%Wing Length Edit Box & Label

%Wing Length Edit Box Label
wlengthtxt_h = uicontrol(...
    'Style','text',...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'BackgroundColor',[0.87 0.87 0.87],...
    'Position',[1-(buttonlength+xButton) .2 buttonlength buttonheight],...
    'FontSize',10,...
    'String','Wing Length'...
    );

%Wing Length Edit Box
wlengthedit_h = uicontrol(...
    'Style','edit',...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'Position',[1-(buttonlength+xButton) .2-ybuttonspace buttonlength buttonheight],...
    'FontSize',10,...
    'String',''...
    );

%-------------------------------------------------------------------------%

%Buttons

%Plot Button
plotbn_h = uicontrol(...
    'Style','pushbutton',...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'Position',[1-(buttonlength+xButton) .2-2*ybuttonspace buttonlength buttonheight],...
    'FontSize',10,...
    'String','Plot Airfoils',...
    'Callback',@plotbuttonfn...
    );

%Export Button
exportbn_h = uicontrol(...
    'Style','pushbutton',...
    'Visible','off',...
    'Units','normalized',...
    'Parent',figure_h,...
    'Position',[1-(buttonlength+xButton) .2-2*ybuttonspace-1.2*buttonheight buttonlength buttonheight],...
    'FontSize',10,...
    'String','Export Airfoils',...
    'Callback',@exportbuttonfn...
    );

%=========================================================================%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% CALLBACK FUNCTIONS %%%

%Plot Button callback function.
function plotbuttonfn(source,eventdata)
    %This function will take all the information from the edit boxes and
    %plot the airfoils. If a text field was mistakenly left empty, the
    %function will terminate without plotting with an error message
    %alerting the user of the missing field(s) and prompt the user to enter
    %values before plotting again.
    
    %Get all values from edit boxes.
    NACAnum1=get(NACA1edit_h,'String');
    NACAnum2=get(NACA2edit_h,'String');
    chord1=str2double(get(chord1edit_h,'String'));
    chord2=str2double(get(chord2edit_h,'String'));
    AOI1=str2double(get(AOI1edit_h,'String'));
    AOI2=str2double(get(AOI2edit_h,'String'));
    d=str2double(get(wlengthedit_h,'String'));
    
    %Calculates coordinates for
    theta1=AOI1*(pi/180);
    theta2=AOI2*(pi/180);
    
    t1=str2double(NACAnum1(3:4))/100; %Max thickness as a fraction of the chord.
    t2=str2double(NACAnum2(3:4))/100; %Max thickness as a fraction of the chord.
    
    m1=str2double(NACAnum1(1))/100; %Max camber.
    m2=str2double(NACAnum2(1))/100; %Max camber.
    
    p1=str2double(NACAnum1(2))/10; %Location of max camber relative to chord length.
    p2=str2double(NACAnum2(2))/10; %Location of max camber relative to chord length.
    
    [X_c1,Y_c1,X_s1,Y_s1]=plotNACA(NACAnum1,chord1,AOI1,plot1_h);
    [X_c2,Y_c2,X_s2,Y_s2]=plotNACA(NACAnum2,chord2,AOI2,plot2_h);
    d_0=zeros(1,length(X_s1));
    d_1=d*ones(1,length(X_s1));
    [X Y Z]=transv(X_s1,Y_s1,X_s2,Y_s2,d);
    
    titlestr1=sprintf('NACA Airfoil %s   Chord: %.1fin   AOI: %.2f%c',NACAnum1,chord1,AOI1,char(176));
    titlestr2=sprintf('NACA Airfoil %s   Chord: %.1fin   AOI: %.2f%c',NACAnum2,chord2,AOI2,char(176));
    
    title(plot1_h,titlestr1);
    title(plot2_h,titlestr2);
    xlabel(plot1_h,'Inches');
    ylabel(plot1_h,'Inches');
    xlabel(plot2_h,'Inches');
    ylabel(plot2_h,'Inches');
    
    axes(plot1_h)
    daspect([1 1 1])
    axes(plot2_h)
    daspect([1 1 1])
    
    axes(plot3_h)
    cla
    h=plot3(d_0,X_s1,Y_s1,d_1,X_s2,Y_s2);
    set(h,'Color','b')
    hold on
    plot3(Z,X,Y);
    axis equal
    title('Isometric Wing View')
end
    
    
function exportbuttonfn(source,eventdata)   

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% INITIALIZING THE GUI %%%


%Make all visible.

set([figure_h plot1_h plot2_h plot3_h NACA1txt_h NACA1edit_h NACA2txt_h NACA2edit_h chord1txt_h chord2txt_h...
    chord1edit_h chord2edit_h AOI1txt_h AOI2txt_h AOI1edit_h AOI2edit_h wlengthtxt_h wlengthedit_h plotbn_h exportbn_h],...
    'Visible','on'...
    );
end